==> wry_bokeh_helper-0.3.1-cp312-cp312-macosx_11_0_arm64.whl: wry_bokeh_helper/bokeh_io.py <==
from __future__ import annotations

import io
import json
import os
import pathlib
import urllib.request
from typing import TYPE_CHECKING, Any, overload

from PIL import Image

from wry_bokeh_helper._wry_bokeh_helper import render_bokeh

if TYPE_CHECKING:
    from wry_bokeh_helper._wry_bokeh_helper import ResourceType

    try:
        from bokeh.embed.standalone import StandaloneEmbedJson
        from bokeh.models import Model

    except ImportError:
        Model = Any
        StandaloneEmbedJson = dict[str, Any]

    BokehFigureOrStandaloneJson = Model | StandaloneEmbedJson


@overload
def bokeh_to_image(
    bokeh_figure_or_bokeh_standalone_json: BokehFigureOrStandaloneJson,
    *,
    resource: tuple[ResourceType, str] | None = None,
) -> Image.Image:
    """
    Converts a Bokeh figure or standalone JSON to an image.

    Args:
        bokeh_figure_or_bokeh_standalone_json (BokehFigureOrStandaloneJson):
            The Bokeh figure or standalone JSON to convert.
        resource (tuple[ResourceType, str] | None, optional):
            Additional resources required for the conversion. Defaults to None.

    Returns:
        Image.Image: The resulting image.
    """
    ...


@overload
def bokeh_to_image(
    bokeh_figure_or_bokeh_standalone_json: BokehFigureOrStandaloneJson,
    filepath: os.PathLike[str] | str,
    *,
    dpi: int = 300,
    resource: tuple[ResourceType, str] | None = None,
) -> None:
    """
    Save a Bokeh plot to a specified file path.

    Parameters:
        bokeh_figure_or_bokeh_standalone_json (BokehFigureOrStandaloneJson):
            The Bokeh figure or standalone JSON to be saved as an image.
        filepath (os.PathLike[str] | str):
            The file path where the image will be saved.
        dpi (int, optional):
            The resolution of the saved image in dots per inch. Default is 300.
        resource (tuple[ResourceType, str] | None, optional):
            Additional resources required for saving the image. Default is None.

    Returns:
        None
    """
    ...


def bokeh_to_image(
    bokeh_figure_or_bokeh_standalone_json: BokehFigureOrStandaloneJson,
    filepath: os.PathLike[str] | str | None = None,
    *,
    dpi: int = 300,
    resource: tuple[ResourceType, str] | None = None,
) -> Image.Image | None:
    if isinstance(bokeh_figure_or_bokeh_standalone_json, dict):
        bokeh_json_item = bokeh_figure_or_bokeh_standalone_json

    else:
        try:
            from bokeh.embed.standalone import json_item
            from bokeh.models import Model
        except ImportError:
            raise ImportError("bokeh is not installed.")
        if not isinstance(bokeh_figure_or_bokeh_standalone_json, Model):
            raise TypeError(
                "bokeh_figure_or_bokeh_standalone_json must be a Bokeh Model."
            )
        bokeh_json_item = json_item(bokeh_figure_or_bokeh_standalone_json)

    png = render_bokeh(json.dumps(bokeh_json_item), dpi, resource)
    response = urllib.request.urlopen(png)
    img = Image.open(io.BytesIO(response.read()))

    if filepath:
        # if want jpg, convert RGBA to RGB
        filepath = pathlib.Path(filepath)
        if filepath.suffix == ".jpg" or filepath.suffix == ".jpeg":
            img = img.convert("RGB")
        return img.save(filepath, dpi=(dpi, dpi))
    return img

==> wry_bokeh_helper-0.3.2-cp312-cp312-macosx_11_0_arm64.whl: wry_bokeh_helper/bokeh_io.py <==
from __future__ import annotations

import io
import json
import os
import pathlib
import sys
import urllib.request
from typing import TYPE_CHECKING, Any, overload

from PIL import Image

from wry_bokeh_helper._wry_bokeh_helper import render_bokeh

if TYPE_CHECKING:
    from multiprocessing import Queue

    from wry_bokeh_helper._wry_bokeh_helper import ResourceType

    try:
        from bokeh.embed.standalone import StandaloneEmbedJson
        from bokeh.models import Model

    except ImportError:
        Model = Any
        StandaloneEmbedJson = dict[str, Any]

    BokehFigureOrStandaloneJson = Model | StandaloneEmbedJson


def _render_bokeh(
    bokeh_json_item: dict[str, Any],
    dpi: int,
    resource: tuple[ResourceType, str] | None,
) -> str:
    try:
        return render_bokeh(
            json.dumps(bokeh_json_item),
            dpi,
            resource,
        )
    except BaseException as e:
        raise e


def _run_in_process(
    queue: Queue,
    bokeh_json_item: dict[str, Any],
    dpi: int,
    resource: tuple[ResourceType, str] | None,
):
    try:
        data_url = _render_bokeh(
            bokeh_json_item,
            dpi,
            resource,
        )
        queue.put(data_url)
    except BaseException as e:
        queue.put(e)


def _get_img_data_url_in_subprocess(
    bokeh_json_item: dict[str, Any],
    dpi: int,
    resource: tuple[ResourceType, str] | None,
) -> str:
    from multiprocessing import Process, Queue, freeze_support
    from multiprocessing.process import current_process
    from queue import Empty

    if getattr(current_process(), "_inheriting", False):
        freeze_support()
    queue: Queue[str | BaseException] = Queue()
    process = Process(
        target=_run_in_process,
        args=(
            queue,
            bokeh_json_item,
            dpi,
            resource,
        ),
    )
    process.start()

    try:
        result = queue.get(timeout=60)
    except Empty:
        raise TimeoutError("The process took too long to complete.")
    if isinstance(result, BaseException):
        raise result
    return result


@overload
def bokeh_to_image(
    bokeh_figure_or_bokeh_standalone_json: BokehFigureOrStandaloneJson,
    *,
    resource: tuple[ResourceType, str] | None = None,
) -> Image.Image:
    """
    Converts a Bokeh figure or standalone JSON to an image.

    Args:
        bokeh_figure_or_bokeh_standalone_json (BokehFigureOrStandaloneJson):
            The Bokeh figure or standalone JSON to convert.
        resource (tuple[ResourceType, str] | None, optional):
            Additional resources required for the conversion. Defaults to None.

    Returns:
        Image.Image: The resulting image.
    """
    ...


@overload
def bokeh_to_image(
    bokeh_figure_or_bokeh_standalone_json: BokehFigureOrStandaloneJson,
    filepath: os.PathLike[str] | str,
    *,
    dpi: int = 300,
    resource: tuple[ResourceType, str] | None = None,
) -> None:
    """
    Save a Bokeh plot to a specified file path.

    Parameters:
        bokeh_figure_or_bokeh_standalone_json (BokehFigureOrStandaloneJson):
            The Bokeh figure or standalone JSON to be saved as an image.
        filepath (os.PathLike[str] | str):
            The file path where the image will be saved.
        dpi (int, optional):
            The resolution of the saved image in dots per inch. Default is 300.
        resource (tuple[ResourceType, str] | None, optional):
            Additional resources required for saving the image. Default is None.

    Returns:
        None
    """
    ...


def bokeh_to_image(
    bokeh_figure_or_bokeh_standalone_json: BokehFigureOrStandaloneJson,
    filepath: os.PathLike[str] | str | None = None,
    *,
    dpi: int = 300,
    resource: tuple[ResourceType, str] | None = None,
) -> Image.Image | None:
    if isinstance(bokeh_figure_or_bokeh_standalone_json, dict):
        bokeh_json_item = bokeh_figure_or_bokeh_standalone_json

    else:
        try:
            from bokeh.embed.standalone import json_item
            from bokeh.models import Model
        except ImportError:
            raise ImportError("bokeh is not installed.")
        if not isinstance(bokeh_figure_or_bokeh_standalone_json, Model):
            raise TypeError(
                "bokeh_figure_or_bokeh_standalone_json must be a Bokeh Model."
            )
        bokeh_json_item = json_item(bokeh_figure_or_bokeh_standalone_json)
    is_MacOS = sys.platform == "darwin"
    if is_MacOS:
        img_data_url = _get_img_data_url_in_subprocess(
            {**bokeh_json_item}, dpi, resource
        )
    else:
        img_data_url = _render_bokeh({**bokeh_json_item}, dpi, resource)
    response = urllib.request.urlopen(img_data_url)
    img = Image.open(io.BytesIO(response.read()))

    if filepath:
        # if want jpg, convert RGBA to RGB
        filepath = pathlib.Path(filepath)
        if filepath.suffix == ".jpg" or filepath.suffix == ".jpeg":
            img = img.convert("RGB")
        return img.save(filepath, dpi=(dpi, dpi))
    return img
